# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.country_coverage_retrieve_response import CountryCoverageRetrieveResponse
from ..types.country_coverage_retrieve_country_response import CountryCoverageRetrieveCountryResponse

__all__ = ["CountryCoverageResource", "AsyncCountryCoverageResource"]


class CountryCoverageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CountryCoverageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CountryCoverageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CountryCoverageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CountryCoverageResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CountryCoverageRetrieveResponse:
        """Get country coverage"""
        return self._get(
            "/country_coverage",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CountryCoverageRetrieveResponse,
        )

    def retrieve_country(
        self,
        country_code: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CountryCoverageRetrieveCountryResponse:
        """
        Get coverage for a specific country

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not country_code:
            raise ValueError(f"Expected a non-empty value for `country_code` but received {country_code!r}")
        return self._get(
            f"/country_coverage/countries/{country_code}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CountryCoverageRetrieveCountryResponse,
        )


class AsyncCountryCoverageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCountryCoverageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCountryCoverageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCountryCoverageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCountryCoverageResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CountryCoverageRetrieveResponse:
        """Get country coverage"""
        return await self._get(
            "/country_coverage",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CountryCoverageRetrieveResponse,
        )

    async def retrieve_country(
        self,
        country_code: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CountryCoverageRetrieveCountryResponse:
        """
        Get coverage for a specific country

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not country_code:
            raise ValueError(f"Expected a non-empty value for `country_code` but received {country_code!r}")
        return await self._get(
            f"/country_coverage/countries/{country_code}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CountryCoverageRetrieveCountryResponse,
        )


class CountryCoverageResourceWithRawResponse:
    def __init__(self, country_coverage: CountryCoverageResource) -> None:
        self._country_coverage = country_coverage

        self.retrieve = to_raw_response_wrapper(
            country_coverage.retrieve,
        )
        self.retrieve_country = to_raw_response_wrapper(
            country_coverage.retrieve_country,
        )


class AsyncCountryCoverageResourceWithRawResponse:
    def __init__(self, country_coverage: AsyncCountryCoverageResource) -> None:
        self._country_coverage = country_coverage

        self.retrieve = async_to_raw_response_wrapper(
            country_coverage.retrieve,
        )
        self.retrieve_country = async_to_raw_response_wrapper(
            country_coverage.retrieve_country,
        )


class CountryCoverageResourceWithStreamingResponse:
    def __init__(self, country_coverage: CountryCoverageResource) -> None:
        self._country_coverage = country_coverage

        self.retrieve = to_streamed_response_wrapper(
            country_coverage.retrieve,
        )
        self.retrieve_country = to_streamed_response_wrapper(
            country_coverage.retrieve_country,
        )


class AsyncCountryCoverageResourceWithStreamingResponse:
    def __init__(self, country_coverage: AsyncCountryCoverageResource) -> None:
        self._country_coverage = country_coverage

        self.retrieve = async_to_streamed_response_wrapper(
            country_coverage.retrieve,
        )
        self.retrieve_country = async_to_streamed_response_wrapper(
            country_coverage.retrieve_country,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .rcs_agent_message import RcsAgentMessage

__all__ = ["MesssageRcsResponse", "Data", "DataFrom", "DataTo"]


class DataFrom(BaseModel):
    agent_id: Optional[str] = None
    """agent ID"""

    agent_name: Optional[str] = None

    carrier: Optional[str] = None


class DataTo(BaseModel):
    carrier: Optional[str] = None

    line_type: Optional[str] = None

    phone_number: Optional[str] = None

    status: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None
    """message ID"""

    body: Optional[RcsAgentMessage] = None

    direction: Optional[str] = None

    encoding: Optional[str] = None

    from_: Optional[DataFrom] = FieldInfo(alias="from", default=None)

    messaging_profile_id: Optional[str] = None

    organization_id: Optional[str] = None

    received_at: Optional[datetime] = None

    record_type: Optional[str] = None

    to: Optional[List[DataTo]] = None

    type: Optional[str] = None


class MesssageRcsResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ClusterRetrieveParams"]


class ClusterRetrieveParams(TypedDict, total=False):
    show_subclusters: bool
    """Whether or not to include subclusters and their nodes in the response."""

    top_n_nodes: int
    """The number of nodes in the cluster to return in the response.

    Nodes will be sorted by their centrality within the cluster.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["MediaFeaturesParam"]


class MediaFeaturesParam(TypedDict, total=False):
    accept_any_rtp_packets_enabled: bool
    """
    When enabled, Telnyx will accept RTP packets from any customer-side IP address
    and port, not just those to which Telnyx is sending RTP.
    """

    rtp_auto_adjust_enabled: bool
    """
    When RTP Auto-Adjust is enabled, the destination RTP address port will be
    automatically changed to match the source of the incoming RTP packets.
    """

    t38_fax_gateway_enabled: bool
    """Controls whether Telnyx will accept a T.38 re-INVITE for this phone number.

    Note that Telnyx will not send a T.38 re-INVITE; this option only controls
    whether one will be accepted.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConnectionListActiveCallsResponse", "Data", "Meta", "MetaCursors"]


class Data(BaseModel):
    call_control_id: str
    """Unique identifier and token for controlling the call."""

    call_duration: int
    """Indicates the duration of the call in seconds"""

    call_leg_id: str
    """ID that is unique to the call and can be used to correlate webhook events"""

    call_session_id: str
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call
    """

    client_state: str
    """State received from a command."""

    record_type: Literal["call"]


class MetaCursors(BaseModel):
    after: Optional[str] = None
    """Opaque identifier of next page."""

    before: Optional[str] = None
    """Opaque identifier of previous page."""


class Meta(BaseModel):
    cursors: Optional[MetaCursors] = None

    next: Optional[str] = None
    """Path to next page."""

    previous: Optional[str] = None
    """Path to previous page."""

    total_items: Optional[int] = None


class ConnectionListActiveCallsResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .portout_report import PortoutReport as PortoutReport
from .event_list_params import EventListParams as EventListParams
from .report_list_params import ReportListParams as ReportListParams
from .event_list_response import EventListResponse as EventListResponse
from .report_create_params import ReportCreateParams as ReportCreateParams
from .report_list_response import ReportListResponse as ReportListResponse
from .comment_create_params import CommentCreateParams as CommentCreateParams
from .comment_list_response import CommentListResponse as CommentListResponse
from .report_create_response import ReportCreateResponse as ReportCreateResponse
from .comment_create_response import CommentCreateResponse as CommentCreateResponse
from .event_retrieve_response import EventRetrieveResponse as EventRetrieveResponse
from .report_retrieve_response import ReportRetrieveResponse as ReportRetrieveResponse
from .export_portouts_csv_report import ExportPortoutsCsvReport as ExportPortoutsCsvReport
from .export_portouts_csv_report_param import ExportPortoutsCsvReportParam as ExportPortoutsCsvReportParam
from .supporting_document_create_params import SupportingDocumentCreateParams as SupportingDocumentCreateParams
from .supporting_document_list_response import SupportingDocumentListResponse as SupportingDocumentListResponse
from .supporting_document_create_response import SupportingDocumentCreateResponse as SupportingDocumentCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    partner_campaign_list_params,
    partner_campaign_update_params,
    partner_campaign_list_shared_by_me_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.telnyx_downstream_campaign import TelnyxDownstreamCampaign
from ..types.partner_campaign_list_response import PartnerCampaignListResponse
from ..types.partner_campaign_list_shared_by_me_response import PartnerCampaignListSharedByMeResponse
from ..types.partner_campaign_retrieve_sharing_status_response import PartnerCampaignRetrieveSharingStatusResponse

__all__ = ["PartnerCampaignsResource", "AsyncPartnerCampaignsResource"]


class PartnerCampaignsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PartnerCampaignsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PartnerCampaignsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PartnerCampaignsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PartnerCampaignsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxDownstreamCampaign:
        """
        Retrieve campaign details by `campaignId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._get(
            f"/partner_campaigns/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxDownstreamCampaign,
        )

    def update(
        self,
        campaign_id: str,
        *,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxDownstreamCampaign:
        """Update campaign details by `campaignId`.

        **Please note:** Only webhook urls are
        editable.

        Args:
          webhook_failover_url: Webhook failover to which campaign status updates are sent.

          webhook_url: Webhook to which campaign status updates are sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._patch(
            f"/partner_campaigns/{campaign_id}",
            body=maybe_transform(
                {
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                partner_campaign_update_params.PartnerCampaignUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxDownstreamCampaign,
        )

    def list(
        self,
        *,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedPhoneNumbersCount",
            "-assignedPhoneNumbersCount",
            "brandDisplayName",
            "-brandDisplayName",
            "tcrBrandId",
            "-tcrBranId",
            "tcrCampaignId",
            "-tcrCampaignId",
            "createdAt",
            "-createdAt",
            "campaignStatus",
            "-campaignStatus",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PartnerCampaignListResponse:
        """
        Retrieve all partner campaigns you have shared to Telnyx in a paginated fashion.

        This endpoint is currently limited to only returning shared campaigns that
        Telnyx has accepted. In other words, shared but pending campaigns are currently
        omitted from the response from this endpoint.

        Args:
          page: The 1-indexed page number to get. The default value is `1`.

          records_per_page: The amount of records per page, limited to between 1 and 500 inclusive. The
              default value is `10`.

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/partner_campaigns",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                    },
                    partner_campaign_list_params.PartnerCampaignListParams,
                ),
            ),
            cast_to=PartnerCampaignListResponse,
        )

    def list_shared_by_me(
        self,
        *,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PartnerCampaignListSharedByMeResponse:
        """
        Get all partner campaigns you have shared to Telnyx in a paginated fashion

        This endpoint is currently limited to only returning shared campaigns that
        Telnyx has accepted. In other words, shared but pending campaigns are currently
        omitted from the response from this endpoint.

        Args:
          page: The 1-indexed page number to get. The default value is `1`.

          records_per_page: The amount of records per page, limited to between 1 and 500 inclusive. The
              default value is `10`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/partnerCampaign/sharedByMe",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "records_per_page": records_per_page,
                    },
                    partner_campaign_list_shared_by_me_params.PartnerCampaignListSharedByMeParams,
                ),
            ),
            cast_to=PartnerCampaignListSharedByMeResponse,
        )

    def retrieve_sharing_status(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PartnerCampaignRetrieveSharingStatusResponse:
        """
        Get Sharing Status

        Args:
          campaign_id: ID of the campaign in question

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return self._get(
            f"/partnerCampaign/{campaign_id}/sharing",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PartnerCampaignRetrieveSharingStatusResponse,
        )


class AsyncPartnerCampaignsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPartnerCampaignsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPartnerCampaignsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPartnerCampaignsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPartnerCampaignsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxDownstreamCampaign:
        """
        Retrieve campaign details by `campaignId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._get(
            f"/partner_campaigns/{campaign_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxDownstreamCampaign,
        )

    async def update(
        self,
        campaign_id: str,
        *,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxDownstreamCampaign:
        """Update campaign details by `campaignId`.

        **Please note:** Only webhook urls are
        editable.

        Args:
          webhook_failover_url: Webhook failover to which campaign status updates are sent.

          webhook_url: Webhook to which campaign status updates are sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._patch(
            f"/partner_campaigns/{campaign_id}",
            body=await async_maybe_transform(
                {
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                partner_campaign_update_params.PartnerCampaignUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxDownstreamCampaign,
        )

    async def list(
        self,
        *,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedPhoneNumbersCount",
            "-assignedPhoneNumbersCount",
            "brandDisplayName",
            "-brandDisplayName",
            "tcrBrandId",
            "-tcrBranId",
            "tcrCampaignId",
            "-tcrCampaignId",
            "createdAt",
            "-createdAt",
            "campaignStatus",
            "-campaignStatus",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PartnerCampaignListResponse:
        """
        Retrieve all partner campaigns you have shared to Telnyx in a paginated fashion.

        This endpoint is currently limited to only returning shared campaigns that
        Telnyx has accepted. In other words, shared but pending campaigns are currently
        omitted from the response from this endpoint.

        Args:
          page: The 1-indexed page number to get. The default value is `1`.

          records_per_page: The amount of records per page, limited to between 1 and 500 inclusive. The
              default value is `10`.

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/partner_campaigns",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                    },
                    partner_campaign_list_params.PartnerCampaignListParams,
                ),
            ),
            cast_to=PartnerCampaignListResponse,
        )

    async def list_shared_by_me(
        self,
        *,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PartnerCampaignListSharedByMeResponse:
        """
        Get all partner campaigns you have shared to Telnyx in a paginated fashion

        This endpoint is currently limited to only returning shared campaigns that
        Telnyx has accepted. In other words, shared but pending campaigns are currently
        omitted from the response from this endpoint.

        Args:
          page: The 1-indexed page number to get. The default value is `1`.

          records_per_page: The amount of records per page, limited to between 1 and 500 inclusive. The
              default value is `10`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/partnerCampaign/sharedByMe",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "records_per_page": records_per_page,
                    },
                    partner_campaign_list_shared_by_me_params.PartnerCampaignListSharedByMeParams,
                ),
            ),
            cast_to=PartnerCampaignListSharedByMeResponse,
        )

    async def retrieve_sharing_status(
        self,
        campaign_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PartnerCampaignRetrieveSharingStatusResponse:
        """
        Get Sharing Status

        Args:
          campaign_id: ID of the campaign in question

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_id:
            raise ValueError(f"Expected a non-empty value for `campaign_id` but received {campaign_id!r}")
        return await self._get(
            f"/partnerCampaign/{campaign_id}/sharing",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PartnerCampaignRetrieveSharingStatusResponse,
        )


class PartnerCampaignsResourceWithRawResponse:
    def __init__(self, partner_campaigns: PartnerCampaignsResource) -> None:
        self._partner_campaigns = partner_campaigns

        self.retrieve = to_raw_response_wrapper(
            partner_campaigns.retrieve,
        )
        self.update = to_raw_response_wrapper(
            partner_campaigns.update,
        )
        self.list = to_raw_response_wrapper(
            partner_campaigns.list,
        )
        self.list_shared_by_me = to_raw_response_wrapper(
            partner_campaigns.list_shared_by_me,
        )
        self.retrieve_sharing_status = to_raw_response_wrapper(
            partner_campaigns.retrieve_sharing_status,
        )


class AsyncPartnerCampaignsResourceWithRawResponse:
    def __init__(self, partner_campaigns: AsyncPartnerCampaignsResource) -> None:
        self._partner_campaigns = partner_campaigns

        self.retrieve = async_to_raw_response_wrapper(
            partner_campaigns.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            partner_campaigns.update,
        )
        self.list = async_to_raw_response_wrapper(
            partner_campaigns.list,
        )
        self.list_shared_by_me = async_to_raw_response_wrapper(
            partner_campaigns.list_shared_by_me,
        )
        self.retrieve_sharing_status = async_to_raw_response_wrapper(
            partner_campaigns.retrieve_sharing_status,
        )


class PartnerCampaignsResourceWithStreamingResponse:
    def __init__(self, partner_campaigns: PartnerCampaignsResource) -> None:
        self._partner_campaigns = partner_campaigns

        self.retrieve = to_streamed_response_wrapper(
            partner_campaigns.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            partner_campaigns.update,
        )
        self.list = to_streamed_response_wrapper(
            partner_campaigns.list,
        )
        self.list_shared_by_me = to_streamed_response_wrapper(
            partner_campaigns.list_shared_by_me,
        )
        self.retrieve_sharing_status = to_streamed_response_wrapper(
            partner_campaigns.retrieve_sharing_status,
        )


class AsyncPartnerCampaignsResourceWithStreamingResponse:
    def __init__(self, partner_campaigns: AsyncPartnerCampaignsResource) -> None:
        self._partner_campaigns = partner_campaigns

        self.retrieve = async_to_streamed_response_wrapper(
            partner_campaigns.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            partner_campaigns.update,
        )
        self.list = async_to_streamed_response_wrapper(
            partner_campaigns.list,
        )
        self.list_shared_by_me = async_to_streamed_response_wrapper(
            partner_campaigns.list_shared_by_me,
        )
        self.retrieve_sharing_status = async_to_streamed_response_wrapper(
            partner_campaigns.retrieve_sharing_status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    CredentialConnectionListResponse,
    CredentialConnectionCreateResponse,
    CredentialConnectionDeleteResponse,
    CredentialConnectionUpdateResponse,
    CredentialConnectionRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCredentialConnections:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
        )
        assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
            active=True,
            anchorsite_override="Latency",
            android_push_credential_id="06b09dfd-7154-4980-8b75-cebf7a9d4f8e",
            default_on_hold_comfort_noise_enabled=False,
            dtmf_type="RFC 2833",
            encode_contact_header_enabled=True,
            encrypted_media="SRTP",
            inbound={
                "ani_number_format": "+E.164",
                "channel_limit": 10,
                "codecs": ["string"],
                "dnis_number_format": "+e164",
                "generate_ringback_tone": True,
                "isup_headers_enabled": True,
                "prack_enabled": True,
                "shaken_stir_enabled": True,
                "sip_compact_headers_enabled": True,
                "timeout_1xx_secs": 10,
                "timeout_2xx_secs": 20,
            },
            ios_push_credential_id="ec0c8e5d-439e-4620-a0c1-9d9c8d02a836",
            onnet_t38_passthrough_enabled=True,
            outbound={
                "ani_override": "always",
                "ani_override_type": "always",
                "call_parking_enabled": True,
                "channel_limit": 10,
                "generate_ringback_tone": True,
                "instant_ringback_enabled": True,
                "localization": "US",
                "outbound_voice_profile_id": "outbound_voice_profile_id",
                "t38_reinvite_source": "customer",
            },
            rtcp_settings={
                "capture_enabled": True,
                "port": "rtcp-mux",
                "report_frequency_secs": 10,
            },
            sip_uri_calling_preference="disabled",
            tags=["tag1", "tag2"],
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.credential_connections.with_raw_response.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = response.parse()
        assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.credential_connections.with_streaming_response.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = response.parse()
            assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.retrieve(
            "id",
        )
        assert_matches_type(CredentialConnectionRetrieveResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.credential_connections.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = response.parse()
        assert_matches_type(CredentialConnectionRetrieveResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.credential_connections.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = response.parse()
            assert_matches_type(CredentialConnectionRetrieveResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.credential_connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.update(
            id="id",
        )
        assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.update(
            id="id",
            active=True,
            anchorsite_override="Latency",
            android_push_credential_id="06b09dfd-7154-4980-8b75-cebf7a9d4f8e",
            connection_name="my name",
            default_on_hold_comfort_noise_enabled=False,
            dtmf_type="RFC 2833",
            encode_contact_header_enabled=True,
            encrypted_media="SRTP",
            inbound={
                "ani_number_format": "+E.164",
                "channel_limit": 10,
                "codecs": ["string"],
                "dnis_number_format": "+e164",
                "generate_ringback_tone": True,
                "isup_headers_enabled": True,
                "prack_enabled": True,
                "shaken_stir_enabled": True,
                "sip_compact_headers_enabled": True,
                "timeout_1xx_secs": 10,
                "timeout_2xx_secs": 20,
            },
            ios_push_credential_id="ec0c8e5d-439e-4620-a0c1-9d9c8d02a836",
            onnet_t38_passthrough_enabled=True,
            outbound={
                "ani_override": "always",
                "ani_override_type": "always",
                "call_parking_enabled": True,
                "channel_limit": 10,
                "generate_ringback_tone": True,
                "instant_ringback_enabled": True,
                "localization": "US",
                "outbound_voice_profile_id": "outbound_voice_profile_id",
                "t38_reinvite_source": "customer",
            },
            password="my123secure456password789",
            rtcp_settings={
                "capture_enabled": True,
                "port": "rtcp-mux",
                "report_frequency_secs": 10,
            },
            sip_uri_calling_preference="disabled",
            tags=["tag1", "tag2"],
            user_name="myusername123",
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.credential_connections.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = response.parse()
        assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.credential_connections.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = response.parse()
            assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.credential_connections.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.list()
        assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.list(
            filter={
                "connection_name": {"contains": "contains"},
                "fqdn": "fqdn",
                "outbound_voice_profile_id": "outbound_voice_profile_id",
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort="connection_name",
        )
        assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.credential_connections.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = response.parse()
        assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.credential_connections.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = response.parse()
            assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        credential_connection = client.credential_connections.delete(
            "id",
        )
        assert_matches_type(CredentialConnectionDeleteResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.credential_connections.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = response.parse()
        assert_matches_type(CredentialConnectionDeleteResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.credential_connections.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = response.parse()
            assert_matches_type(CredentialConnectionDeleteResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.credential_connections.with_raw_response.delete(
                "",
            )


class TestAsyncCredentialConnections:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
        )
        assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
            active=True,
            anchorsite_override="Latency",
            android_push_credential_id="06b09dfd-7154-4980-8b75-cebf7a9d4f8e",
            default_on_hold_comfort_noise_enabled=False,
            dtmf_type="RFC 2833",
            encode_contact_header_enabled=True,
            encrypted_media="SRTP",
            inbound={
                "ani_number_format": "+E.164",
                "channel_limit": 10,
                "codecs": ["string"],
                "dnis_number_format": "+e164",
                "generate_ringback_tone": True,
                "isup_headers_enabled": True,
                "prack_enabled": True,
                "shaken_stir_enabled": True,
                "sip_compact_headers_enabled": True,
                "timeout_1xx_secs": 10,
                "timeout_2xx_secs": 20,
            },
            ios_push_credential_id="ec0c8e5d-439e-4620-a0c1-9d9c8d02a836",
            onnet_t38_passthrough_enabled=True,
            outbound={
                "ani_override": "always",
                "ani_override_type": "always",
                "call_parking_enabled": True,
                "channel_limit": 10,
                "generate_ringback_tone": True,
                "instant_ringback_enabled": True,
                "localization": "US",
                "outbound_voice_profile_id": "outbound_voice_profile_id",
                "t38_reinvite_source": "customer",
            },
            rtcp_settings={
                "capture_enabled": True,
                "port": "rtcp-mux",
                "report_frequency_secs": 10,
            },
            sip_uri_calling_preference="disabled",
            tags=["tag1", "tag2"],
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.credential_connections.with_raw_response.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = await response.parse()
        assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.credential_connections.with_streaming_response.create(
            connection_name="my name",
            password="my123secure456password789",
            user_name="myusername123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = await response.parse()
            assert_matches_type(CredentialConnectionCreateResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.retrieve(
            "id",
        )
        assert_matches_type(CredentialConnectionRetrieveResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.credential_connections.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = await response.parse()
        assert_matches_type(CredentialConnectionRetrieveResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.credential_connections.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = await response.parse()
            assert_matches_type(CredentialConnectionRetrieveResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.credential_connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.update(
            id="id",
        )
        assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.update(
            id="id",
            active=True,
            anchorsite_override="Latency",
            android_push_credential_id="06b09dfd-7154-4980-8b75-cebf7a9d4f8e",
            connection_name="my name",
            default_on_hold_comfort_noise_enabled=False,
            dtmf_type="RFC 2833",
            encode_contact_header_enabled=True,
            encrypted_media="SRTP",
            inbound={
                "ani_number_format": "+E.164",
                "channel_limit": 10,
                "codecs": ["string"],
                "dnis_number_format": "+e164",
                "generate_ringback_tone": True,
                "isup_headers_enabled": True,
                "prack_enabled": True,
                "shaken_stir_enabled": True,
                "sip_compact_headers_enabled": True,
                "timeout_1xx_secs": 10,
                "timeout_2xx_secs": 20,
            },
            ios_push_credential_id="ec0c8e5d-439e-4620-a0c1-9d9c8d02a836",
            onnet_t38_passthrough_enabled=True,
            outbound={
                "ani_override": "always",
                "ani_override_type": "always",
                "call_parking_enabled": True,
                "channel_limit": 10,
                "generate_ringback_tone": True,
                "instant_ringback_enabled": True,
                "localization": "US",
                "outbound_voice_profile_id": "outbound_voice_profile_id",
                "t38_reinvite_source": "customer",
            },
            password="my123secure456password789",
            rtcp_settings={
                "capture_enabled": True,
                "port": "rtcp-mux",
                "report_frequency_secs": 10,
            },
            sip_uri_calling_preference="disabled",
            tags=["tag1", "tag2"],
            user_name="myusername123",
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.credential_connections.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = await response.parse()
        assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.credential_connections.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = await response.parse()
            assert_matches_type(CredentialConnectionUpdateResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.credential_connections.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.list()
        assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.list(
            filter={
                "connection_name": {"contains": "contains"},
                "fqdn": "fqdn",
                "outbound_voice_profile_id": "outbound_voice_profile_id",
            },
            page={
                "number": 1,
                "size": 1,
            },
            sort="connection_name",
        )
        assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.credential_connections.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = await response.parse()
        assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.credential_connections.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = await response.parse()
            assert_matches_type(CredentialConnectionListResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        credential_connection = await async_client.credential_connections.delete(
            "id",
        )
        assert_matches_type(CredentialConnectionDeleteResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.credential_connections.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        credential_connection = await response.parse()
        assert_matches_type(CredentialConnectionDeleteResponse, credential_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.credential_connections.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            credential_connection = await response.parse()
            assert_matches_type(CredentialConnectionDeleteResponse, credential_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.credential_connections.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import (
    number_order_phone_number_list_params,
    number_order_phone_number_update_requirements_params,
    number_order_phone_number_update_requirement_group_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.update_regulatory_requirement_param import UpdateRegulatoryRequirementParam
from ..types.number_order_phone_number_list_response import NumberOrderPhoneNumberListResponse
from ..types.number_order_phone_number_retrieve_response import NumberOrderPhoneNumberRetrieveResponse
from ..types.number_order_phone_number_update_requirements_response import (
    NumberOrderPhoneNumberUpdateRequirementsResponse,
)
from ..types.number_order_phone_number_update_requirement_group_response import (
    NumberOrderPhoneNumberUpdateRequirementGroupResponse,
)

__all__ = ["NumberOrderPhoneNumbersResource", "AsyncNumberOrderPhoneNumbersResource"]


class NumberOrderPhoneNumbersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NumberOrderPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NumberOrderPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NumberOrderPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NumberOrderPhoneNumbersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        number_order_phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberRetrieveResponse:
        """
        Get an existing phone number in number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_phone_number_id:
            raise ValueError(
                f"Expected a non-empty value for `number_order_phone_number_id` but received {number_order_phone_number_id!r}"
            )
        return self._get(
            f"/number_order_phone_numbers/{number_order_phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderPhoneNumberRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: number_order_phone_number_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberListResponse:
        """
        Get a list of phone numbers associated to orders.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[country_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/number_order_phone_numbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"filter": filter}, number_order_phone_number_list_params.NumberOrderPhoneNumberListParams
                ),
            ),
            cast_to=NumberOrderPhoneNumberListResponse,
        )

    def update_requirement_group(
        self,
        id: str,
        *,
        requirement_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberUpdateRequirementGroupResponse:
        """
        Update requirement group for a phone number order

        Args:
          requirement_group_id: The ID of the requirement group to associate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/number_order_phone_numbers/{id}/requirement_group",
            body=maybe_transform(
                {"requirement_group_id": requirement_group_id},
                number_order_phone_number_update_requirement_group_params.NumberOrderPhoneNumberUpdateRequirementGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderPhoneNumberUpdateRequirementGroupResponse,
        )

    def update_requirements(
        self,
        number_order_phone_number_id: str,
        *,
        regulatory_requirements: Iterable[UpdateRegulatoryRequirementParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberUpdateRequirementsResponse:
        """
        Updates requirements for a single phone number within a number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_phone_number_id:
            raise ValueError(
                f"Expected a non-empty value for `number_order_phone_number_id` but received {number_order_phone_number_id!r}"
            )
        return self._patch(
            f"/number_order_phone_numbers/{number_order_phone_number_id}",
            body=maybe_transform(
                {"regulatory_requirements": regulatory_requirements},
                number_order_phone_number_update_requirements_params.NumberOrderPhoneNumberUpdateRequirementsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderPhoneNumberUpdateRequirementsResponse,
        )


class AsyncNumberOrderPhoneNumbersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNumberOrderPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNumberOrderPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNumberOrderPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNumberOrderPhoneNumbersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        number_order_phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberRetrieveResponse:
        """
        Get an existing phone number in number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_phone_number_id:
            raise ValueError(
                f"Expected a non-empty value for `number_order_phone_number_id` but received {number_order_phone_number_id!r}"
            )
        return await self._get(
            f"/number_order_phone_numbers/{number_order_phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderPhoneNumberRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: number_order_phone_number_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberListResponse:
        """
        Get a list of phone numbers associated to orders.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[country_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/number_order_phone_numbers",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, number_order_phone_number_list_params.NumberOrderPhoneNumberListParams
                ),
            ),
            cast_to=NumberOrderPhoneNumberListResponse,
        )

    async def update_requirement_group(
        self,
        id: str,
        *,
        requirement_group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberUpdateRequirementGroupResponse:
        """
        Update requirement group for a phone number order

        Args:
          requirement_group_id: The ID of the requirement group to associate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/number_order_phone_numbers/{id}/requirement_group",
            body=await async_maybe_transform(
                {"requirement_group_id": requirement_group_id},
                number_order_phone_number_update_requirement_group_params.NumberOrderPhoneNumberUpdateRequirementGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderPhoneNumberUpdateRequirementGroupResponse,
        )

    async def update_requirements(
        self,
        number_order_phone_number_id: str,
        *,
        regulatory_requirements: Iterable[UpdateRegulatoryRequirementParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderPhoneNumberUpdateRequirementsResponse:
        """
        Updates requirements for a single phone number within a number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_phone_number_id:
            raise ValueError(
                f"Expected a non-empty value for `number_order_phone_number_id` but received {number_order_phone_number_id!r}"
            )
        return await self._patch(
            f"/number_order_phone_numbers/{number_order_phone_number_id}",
            body=await async_maybe_transform(
                {"regulatory_requirements": regulatory_requirements},
                number_order_phone_number_update_requirements_params.NumberOrderPhoneNumberUpdateRequirementsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderPhoneNumberUpdateRequirementsResponse,
        )


class NumberOrderPhoneNumbersResourceWithRawResponse:
    def __init__(self, number_order_phone_numbers: NumberOrderPhoneNumbersResource) -> None:
        self._number_order_phone_numbers = number_order_phone_numbers

        self.retrieve = to_raw_response_wrapper(
            number_order_phone_numbers.retrieve,
        )
        self.list = to_raw_response_wrapper(
            number_order_phone_numbers.list,
        )
        self.update_requirement_group = to_raw_response_wrapper(
            number_order_phone_numbers.update_requirement_group,
        )
        self.update_requirements = to_raw_response_wrapper(
            number_order_phone_numbers.update_requirements,
        )


class AsyncNumberOrderPhoneNumbersResourceWithRawResponse:
    def __init__(self, number_order_phone_numbers: AsyncNumberOrderPhoneNumbersResource) -> None:
        self._number_order_phone_numbers = number_order_phone_numbers

        self.retrieve = async_to_raw_response_wrapper(
            number_order_phone_numbers.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            number_order_phone_numbers.list,
        )
        self.update_requirement_group = async_to_raw_response_wrapper(
            number_order_phone_numbers.update_requirement_group,
        )
        self.update_requirements = async_to_raw_response_wrapper(
            number_order_phone_numbers.update_requirements,
        )


class NumberOrderPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, number_order_phone_numbers: NumberOrderPhoneNumbersResource) -> None:
        self._number_order_phone_numbers = number_order_phone_numbers

        self.retrieve = to_streamed_response_wrapper(
            number_order_phone_numbers.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            number_order_phone_numbers.list,
        )
        self.update_requirement_group = to_streamed_response_wrapper(
            number_order_phone_numbers.update_requirement_group,
        )
        self.update_requirements = to_streamed_response_wrapper(
            number_order_phone_numbers.update_requirements,
        )


class AsyncNumberOrderPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, number_order_phone_numbers: AsyncNumberOrderPhoneNumbersResource) -> None:
        self._number_order_phone_numbers = number_order_phone_numbers

        self.retrieve = async_to_streamed_response_wrapper(
            number_order_phone_numbers.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            number_order_phone_numbers.list,
        )
        self.update_requirement_group = async_to_streamed_response_wrapper(
            number_order_phone_numbers.update_requirement_group,
        )
        self.update_requirements = async_to_streamed_response_wrapper(
            number_order_phone_numbers.update_requirements,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, Annotated, TypeAlias, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo

__all__ = ["DocumentUploadJsonParams", "DocServiceDocumentUploadURL", "DocServiceDocumentUploadInline"]


class DocServiceDocumentUploadURL(TypedDict, total=False):
    url: Required[str]
    """
    If the file is already hosted publicly, you can provide a URL and have the
    documents service fetch it for you.
    """

    customer_reference: str
    """Optional reference string for customer tracking."""

    filename: str
    """The filename of the document."""


class DocServiceDocumentUploadInline(TypedDict, total=False):
    file: Required[Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]]
    """The Base64 encoded contents of the file you are uploading."""

    customer_reference: str
    """A customer reference string for customer look ups."""

    filename: str
    """The filename of the document."""


DocumentUploadJsonParams: TypeAlias = Union[DocServiceDocumentUploadURL, DocServiceDocumentUploadInline]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CustomSipHeaderParam"]


class CustomSipHeaderParam(TypedDict, total=False):
    name: Required[str]
    """The name of the header to add."""

    value: Required[str]
    """The value of the header."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    telephony_credential_list_params,
    telephony_credential_create_params,
    telephony_credential_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.telephony_credential_list_response import TelephonyCredentialListResponse
from ..types.telephony_credential_create_response import TelephonyCredentialCreateResponse
from ..types.telephony_credential_delete_response import TelephonyCredentialDeleteResponse
from ..types.telephony_credential_update_response import TelephonyCredentialUpdateResponse
from ..types.telephony_credential_retrieve_response import TelephonyCredentialRetrieveResponse

__all__ = ["TelephonyCredentialsResource", "AsyncTelephonyCredentialsResource"]


class TelephonyCredentialsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TelephonyCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TelephonyCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TelephonyCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TelephonyCredentialsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        connection_id: str,
        expires_at: str | Omit = omit,
        name: str | Omit = omit,
        tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialCreateResponse:
        """
        Create a credential.

        Args:
          connection_id: Identifies the Credential Connection this credential is associated with.

          expires_at: ISO-8601 formatted date indicating when the credential will expire.

          tag: Tags a credential. A single tag can hold at maximum 1000 credentials.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/telephony_credentials",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "expires_at": expires_at,
                    "name": name,
                    "tag": tag,
                },
                telephony_credential_create_params.TelephonyCredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialRetrieveResponse:
        """
        Get the details of an existing On-demand Credential.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/telephony_credentials/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        connection_id: str | Omit = omit,
        expires_at: str | Omit = omit,
        name: str | Omit = omit,
        tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialUpdateResponse:
        """
        Update an existing credential.

        Args:
          connection_id: Identifies the Credential Connection this credential is associated with.

          expires_at: ISO-8601 formatted date indicating when the credential will expire.

          tag: Tags a credential. A single tag can hold at maximum 1000 credentials.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/telephony_credentials/{id}",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "expires_at": expires_at,
                    "name": name,
                    "tag": tag,
                },
                telephony_credential_update_params.TelephonyCredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialUpdateResponse,
        )

    def list(
        self,
        *,
        filter: telephony_credential_list_params.Filter | Omit = omit,
        page: telephony_credential_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialListResponse:
        """
        List all On-demand Credentials.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[tag],
              filter[name], filter[status], filter[resource_id], filter[sip_username]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/telephony_credentials",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    telephony_credential_list_params.TelephonyCredentialListParams,
                ),
            ),
            cast_to=TelephonyCredentialListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialDeleteResponse:
        """
        Delete an existing credential.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/telephony_credentials/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialDeleteResponse,
        )

    def create_token(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Create an Access Token (JWT) for the credential.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._post(
            f"/telephony_credentials/{id}/token",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncTelephonyCredentialsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTelephonyCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTelephonyCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTelephonyCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTelephonyCredentialsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        connection_id: str,
        expires_at: str | Omit = omit,
        name: str | Omit = omit,
        tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialCreateResponse:
        """
        Create a credential.

        Args:
          connection_id: Identifies the Credential Connection this credential is associated with.

          expires_at: ISO-8601 formatted date indicating when the credential will expire.

          tag: Tags a credential. A single tag can hold at maximum 1000 credentials.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/telephony_credentials",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "expires_at": expires_at,
                    "name": name,
                    "tag": tag,
                },
                telephony_credential_create_params.TelephonyCredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialRetrieveResponse:
        """
        Get the details of an existing On-demand Credential.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/telephony_credentials/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        connection_id: str | Omit = omit,
        expires_at: str | Omit = omit,
        name: str | Omit = omit,
        tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialUpdateResponse:
        """
        Update an existing credential.

        Args:
          connection_id: Identifies the Credential Connection this credential is associated with.

          expires_at: ISO-8601 formatted date indicating when the credential will expire.

          tag: Tags a credential. A single tag can hold at maximum 1000 credentials.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/telephony_credentials/{id}",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "expires_at": expires_at,
                    "name": name,
                    "tag": tag,
                },
                telephony_credential_update_params.TelephonyCredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: telephony_credential_list_params.Filter | Omit = omit,
        page: telephony_credential_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialListResponse:
        """
        List all On-demand Credentials.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[tag],
              filter[name], filter[status], filter[resource_id], filter[sip_username]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/telephony_credentials",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    telephony_credential_list_params.TelephonyCredentialListParams,
                ),
            ),
            cast_to=TelephonyCredentialListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelephonyCredentialDeleteResponse:
        """
        Delete an existing credential.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/telephony_credentials/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelephonyCredentialDeleteResponse,
        )

    async def create_token(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Create an Access Token (JWT) for the credential.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._post(
            f"/telephony_credentials/{id}/token",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class TelephonyCredentialsResourceWithRawResponse:
    def __init__(self, telephony_credentials: TelephonyCredentialsResource) -> None:
        self._telephony_credentials = telephony_credentials

        self.create = to_raw_response_wrapper(
            telephony_credentials.create,
        )
        self.retrieve = to_raw_response_wrapper(
            telephony_credentials.retrieve,
        )
        self.update = to_raw_response_wrapper(
            telephony_credentials.update,
        )
        self.list = to_raw_response_wrapper(
            telephony_credentials.list,
        )
        self.delete = to_raw_response_wrapper(
            telephony_credentials.delete,
        )
        self.create_token = to_raw_response_wrapper(
            telephony_credentials.create_token,
        )


class AsyncTelephonyCredentialsResourceWithRawResponse:
    def __init__(self, telephony_credentials: AsyncTelephonyCredentialsResource) -> None:
        self._telephony_credentials = telephony_credentials

        self.create = async_to_raw_response_wrapper(
            telephony_credentials.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            telephony_credentials.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            telephony_credentials.update,
        )
        self.list = async_to_raw_response_wrapper(
            telephony_credentials.list,
        )
        self.delete = async_to_raw_response_wrapper(
            telephony_credentials.delete,
        )
        self.create_token = async_to_raw_response_wrapper(
            telephony_credentials.create_token,
        )


class TelephonyCredentialsResourceWithStreamingResponse:
    def __init__(self, telephony_credentials: TelephonyCredentialsResource) -> None:
        self._telephony_credentials = telephony_credentials

        self.create = to_streamed_response_wrapper(
            telephony_credentials.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            telephony_credentials.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            telephony_credentials.update,
        )
        self.list = to_streamed_response_wrapper(
            telephony_credentials.list,
        )
        self.delete = to_streamed_response_wrapper(
            telephony_credentials.delete,
        )
        self.create_token = to_streamed_response_wrapper(
            telephony_credentials.create_token,
        )


class AsyncTelephonyCredentialsResourceWithStreamingResponse:
    def __init__(self, telephony_credentials: AsyncTelephonyCredentialsResource) -> None:
        self._telephony_credentials = telephony_credentials

        self.create = async_to_streamed_response_wrapper(
            telephony_credentials.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            telephony_credentials.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            telephony_credentials.update,
        )
        self.list = async_to_streamed_response_wrapper(
            telephony_credentials.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            telephony_credentials.delete,
        )
        self.create_token = async_to_streamed_response_wrapper(
            telephony_credentials.create_token,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .rcs_suggestion_param import RcsSuggestionParam
from .rcs_card_content_param import RcsCardContentParam
from .rcs_content_info_param import RcsContentInfoParam

__all__ = [
    "RcsAgentMessageParam",
    "ContentMessage",
    "ContentMessageRichCard",
    "ContentMessageRichCardCarouselCard",
    "ContentMessageRichCardStandaloneCard",
    "Event",
]


class ContentMessageRichCardCarouselCard(TypedDict, total=False):
    card_contents: Required[Iterable[RcsCardContentParam]]
    """The list of contents for each card in the carousel.

    A carousel can have a minimum of 2 cards and a maximum 10 cards.
    """

    card_width: Required[Literal["CARD_WIDTH_UNSPECIFIED", "SMALL", "MEDIUM"]]
    """The width of the cards in the carousel."""


class ContentMessageRichCardStandaloneCard(TypedDict, total=False):
    card_content: Required[RcsCardContentParam]

    card_orientation: Required[Literal["CARD_ORIENTATION_UNSPECIFIED", "HORIZONTAL", "VERTICAL"]]
    """Orientation of the card."""

    thumbnail_image_alignment: Required[Literal["THUMBNAIL_IMAGE_ALIGNMENT_UNSPECIFIED", "LEFT", "RIGHT"]]
    """Image preview alignment for standalone cards with horizontal layout."""


class ContentMessageRichCard(TypedDict, total=False):
    carousel_card: ContentMessageRichCardCarouselCard
    """Carousel of cards."""

    standalone_card: ContentMessageRichCardStandaloneCard
    """Standalone card"""


class ContentMessage(TypedDict, total=False):
    content_info: RcsContentInfoParam

    rich_card: ContentMessageRichCard

    suggestions: Iterable[RcsSuggestionParam]
    """List of suggested actions and replies"""

    text: str
    """Text (maximum 3072 characters)"""


class Event(TypedDict, total=False):
    event_type: Literal["TYPE_UNSPECIFIED", "IS_TYPING", "READ"]


class RcsAgentMessageParam(TypedDict, total=False):
    content_message: ContentMessage

    event: Event
    """RCS Event to send to the recipient"""

    expire_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Timestamp in UTC of when this message is considered expired"""

    ttl: str
    """Duration in seconds ending with 's'"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .inference_embedding_transfer_tool_params_param import InferenceEmbeddingTransferToolParamsParam

__all__ = ["TransferToolParam"]


class TransferToolParam(TypedDict, total=False):
    transfer: Required[InferenceEmbeddingTransferToolParamsParam]

    type: Required[Literal["transfer"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    virtual_cross_connect_list_params,
    virtual_cross_connect_create_params,
    virtual_cross_connect_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.virtual_cross_connect_list_response import VirtualCrossConnectListResponse
from ..types.virtual_cross_connect_create_response import VirtualCrossConnectCreateResponse
from ..types.virtual_cross_connect_delete_response import VirtualCrossConnectDeleteResponse
from ..types.virtual_cross_connect_update_response import VirtualCrossConnectUpdateResponse
from ..types.virtual_cross_connect_retrieve_response import VirtualCrossConnectRetrieveResponse

__all__ = ["VirtualCrossConnectsResource", "AsyncVirtualCrossConnectsResource"]


class VirtualCrossConnectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VirtualCrossConnectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VirtualCrossConnectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VirtualCrossConnectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VirtualCrossConnectsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        bgp_asn: float,
        cloud_provider: Literal["aws", "azure", "gce"],
        cloud_provider_region: str,
        network_id: str,
        primary_cloud_account_id: str,
        region_code: str,
        bandwidth_mbps: float | Omit = omit,
        name: str | Omit = omit,
        primary_bgp_key: str | Omit = omit,
        primary_cloud_ip: str | Omit = omit,
        primary_telnyx_ip: str | Omit = omit,
        secondary_bgp_key: str | Omit = omit,
        secondary_cloud_account_id: str | Omit = omit,
        secondary_cloud_ip: str | Omit = omit,
        secondary_telnyx_ip: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectCreateResponse:
        """
        Create a new Virtual Cross Connect.<br /><br />For AWS and GCE, you have the
        option of creating the primary connection first and the secondary connection
        later. You also have the option of disabling the primary and/or secondary
        connections at any time and later re-enabling them. With Azure, you do not have
        this option. Azure requires both the primary and secondary connections to be
        created at the same time and they can not be independantly disabled.

        Args:
          bgp_asn: The Border Gateway Protocol (BGP) Autonomous System Number (ASN). If null, value
              will be assigned by Telnyx.

          cloud_provider: The Virtual Private Cloud with which you would like to establish a cross
              connect.

          cloud_provider_region: The region where your Virtual Private Cloud hosts are located.<br /><br />The
              available regions can be found using the /virtual_cross_connect_regions
              endpoint.

          network_id: The id of the network associated with the interface.

          primary_cloud_account_id: The identifier for your Virtual Private Cloud. The number will be different
              based upon your Cloud provider.

          region_code: The region the interface should be deployed to.

          bandwidth_mbps: The desired throughput in Megabits per Second (Mbps) for your Virtual Cross
              Connect.<br /><br />The available bandwidths can be found using the
              /virtual_cross_connect_regions endpoint.

          name: A user specified name for the interface.

          primary_bgp_key: The authentication key for BGP peer configuration.

          primary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          primary_telnyx_ip: The IP address assigned to the Telnyx side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          secondary_bgp_key: The authentication key for BGP peer configuration.

          secondary_cloud_account_id: The identifier for your Virtual Private Cloud. The number will be different
              based upon your Cloud provider.<br /><br />This attribute is only necessary for
              GCE.

          secondary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          secondary_telnyx_ip: The IP address assigned to the Telnyx side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/virtual_cross_connects",
            body=maybe_transform(
                {
                    "bgp_asn": bgp_asn,
                    "cloud_provider": cloud_provider,
                    "cloud_provider_region": cloud_provider_region,
                    "network_id": network_id,
                    "primary_cloud_account_id": primary_cloud_account_id,
                    "region_code": region_code,
                    "bandwidth_mbps": bandwidth_mbps,
                    "name": name,
                    "primary_bgp_key": primary_bgp_key,
                    "primary_cloud_ip": primary_cloud_ip,
                    "primary_telnyx_ip": primary_telnyx_ip,
                    "secondary_bgp_key": secondary_bgp_key,
                    "secondary_cloud_account_id": secondary_cloud_account_id,
                    "secondary_cloud_ip": secondary_cloud_ip,
                    "secondary_telnyx_ip": secondary_telnyx_ip,
                },
                virtual_cross_connect_create_params.VirtualCrossConnectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectRetrieveResponse:
        """
        Retrieve a Virtual Cross Connect.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/virtual_cross_connects/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        primary_cloud_ip: str | Omit = omit,
        primary_enabled: bool | Omit = omit,
        primary_routing_announcement: bool | Omit = omit,
        secondary_cloud_ip: str | Omit = omit,
        secondary_enabled: bool | Omit = omit,
        secondary_routing_announcement: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectUpdateResponse:
        """
        Update the Virtual Cross Connect.<br /><br />Cloud IPs can only be patched
        during the `created` state, as GCE will only inform you of your generated IP
        once the pending connection requested has been accepted. Once the Virtual Cross
        Connect has moved to `provisioning`, the IPs can no longer be
        patched.<br /><br />Once the Virtual Cross Connect has moved to `provisioned`
        and you are ready to enable routing, you can toggle the routing announcements to
        `true`.

        Args:
          primary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value can not be patched once the VXC has bene provisioned.

          primary_enabled: Indicates whether the primary circuit is enabled. Setting this to `false` will
              disable the circuit.

          primary_routing_announcement: Whether the primary BGP route is being announced.

          secondary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value can not be patched once the VXC has bene provisioned.

          secondary_enabled: Indicates whether the secondary circuit is enabled. Setting this to `false` will
              disable the circuit.

          secondary_routing_announcement: Whether the secondary BGP route is being announced.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/virtual_cross_connects/{id}",
            body=maybe_transform(
                {
                    "primary_cloud_ip": primary_cloud_ip,
                    "primary_enabled": primary_enabled,
                    "primary_routing_announcement": primary_routing_announcement,
                    "secondary_cloud_ip": secondary_cloud_ip,
                    "secondary_enabled": secondary_enabled,
                    "secondary_routing_announcement": secondary_routing_announcement,
                },
                virtual_cross_connect_update_params.VirtualCrossConnectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectUpdateResponse,
        )

    def list(
        self,
        *,
        filter: virtual_cross_connect_list_params.Filter | Omit = omit,
        page: virtual_cross_connect_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectListResponse:
        """
        List all Virtual Cross Connects.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[network_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/virtual_cross_connects",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    virtual_cross_connect_list_params.VirtualCrossConnectListParams,
                ),
            ),
            cast_to=VirtualCrossConnectListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectDeleteResponse:
        """
        Delete a Virtual Cross Connect.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/virtual_cross_connects/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectDeleteResponse,
        )


class AsyncVirtualCrossConnectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVirtualCrossConnectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVirtualCrossConnectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVirtualCrossConnectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVirtualCrossConnectsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        bgp_asn: float,
        cloud_provider: Literal["aws", "azure", "gce"],
        cloud_provider_region: str,
        network_id: str,
        primary_cloud_account_id: str,
        region_code: str,
        bandwidth_mbps: float | Omit = omit,
        name: str | Omit = omit,
        primary_bgp_key: str | Omit = omit,
        primary_cloud_ip: str | Omit = omit,
        primary_telnyx_ip: str | Omit = omit,
        secondary_bgp_key: str | Omit = omit,
        secondary_cloud_account_id: str | Omit = omit,
        secondary_cloud_ip: str | Omit = omit,
        secondary_telnyx_ip: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectCreateResponse:
        """
        Create a new Virtual Cross Connect.<br /><br />For AWS and GCE, you have the
        option of creating the primary connection first and the secondary connection
        later. You also have the option of disabling the primary and/or secondary
        connections at any time and later re-enabling them. With Azure, you do not have
        this option. Azure requires both the primary and secondary connections to be
        created at the same time and they can not be independantly disabled.

        Args:
          bgp_asn: The Border Gateway Protocol (BGP) Autonomous System Number (ASN). If null, value
              will be assigned by Telnyx.

          cloud_provider: The Virtual Private Cloud with which you would like to establish a cross
              connect.

          cloud_provider_region: The region where your Virtual Private Cloud hosts are located.<br /><br />The
              available regions can be found using the /virtual_cross_connect_regions
              endpoint.

          network_id: The id of the network associated with the interface.

          primary_cloud_account_id: The identifier for your Virtual Private Cloud. The number will be different
              based upon your Cloud provider.

          region_code: The region the interface should be deployed to.

          bandwidth_mbps: The desired throughput in Megabits per Second (Mbps) for your Virtual Cross
              Connect.<br /><br />The available bandwidths can be found using the
              /virtual_cross_connect_regions endpoint.

          name: A user specified name for the interface.

          primary_bgp_key: The authentication key for BGP peer configuration.

          primary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          primary_telnyx_ip: The IP address assigned to the Telnyx side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          secondary_bgp_key: The authentication key for BGP peer configuration.

          secondary_cloud_account_id: The identifier for your Virtual Private Cloud. The number will be different
              based upon your Cloud provider.<br /><br />This attribute is only necessary for
              GCE.

          secondary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          secondary_telnyx_ip: The IP address assigned to the Telnyx side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value should be null for GCE as Google will only inform you
              of your assigned IP once the connection has been accepted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/virtual_cross_connects",
            body=await async_maybe_transform(
                {
                    "bgp_asn": bgp_asn,
                    "cloud_provider": cloud_provider,
                    "cloud_provider_region": cloud_provider_region,
                    "network_id": network_id,
                    "primary_cloud_account_id": primary_cloud_account_id,
                    "region_code": region_code,
                    "bandwidth_mbps": bandwidth_mbps,
                    "name": name,
                    "primary_bgp_key": primary_bgp_key,
                    "primary_cloud_ip": primary_cloud_ip,
                    "primary_telnyx_ip": primary_telnyx_ip,
                    "secondary_bgp_key": secondary_bgp_key,
                    "secondary_cloud_account_id": secondary_cloud_account_id,
                    "secondary_cloud_ip": secondary_cloud_ip,
                    "secondary_telnyx_ip": secondary_telnyx_ip,
                },
                virtual_cross_connect_create_params.VirtualCrossConnectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectRetrieveResponse:
        """
        Retrieve a Virtual Cross Connect.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/virtual_cross_connects/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        primary_cloud_ip: str | Omit = omit,
        primary_enabled: bool | Omit = omit,
        primary_routing_announcement: bool | Omit = omit,
        secondary_cloud_ip: str | Omit = omit,
        secondary_enabled: bool | Omit = omit,
        secondary_routing_announcement: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectUpdateResponse:
        """
        Update the Virtual Cross Connect.<br /><br />Cloud IPs can only be patched
        during the `created` state, as GCE will only inform you of your generated IP
        once the pending connection requested has been accepted. Once the Virtual Cross
        Connect has moved to `provisioning`, the IPs can no longer be
        patched.<br /><br />Once the Virtual Cross Connect has moved to `provisioned`
        and you are ready to enable routing, you can toggle the routing announcements to
        `true`.

        Args:
          primary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value can not be patched once the VXC has bene provisioned.

          primary_enabled: Indicates whether the primary circuit is enabled. Setting this to `false` will
              disable the circuit.

          primary_routing_announcement: Whether the primary BGP route is being announced.

          secondary_cloud_ip: The IP address assigned for your side of the Virtual Cross
              Connect.<br /><br />If none is provided, one will be generated for
              you.<br /><br />This value can not be patched once the VXC has bene provisioned.

          secondary_enabled: Indicates whether the secondary circuit is enabled. Setting this to `false` will
              disable the circuit.

          secondary_routing_announcement: Whether the secondary BGP route is being announced.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/virtual_cross_connects/{id}",
            body=await async_maybe_transform(
                {
                    "primary_cloud_ip": primary_cloud_ip,
                    "primary_enabled": primary_enabled,
                    "primary_routing_announcement": primary_routing_announcement,
                    "secondary_cloud_ip": secondary_cloud_ip,
                    "secondary_enabled": secondary_enabled,
                    "secondary_routing_announcement": secondary_routing_announcement,
                },
                virtual_cross_connect_update_params.VirtualCrossConnectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: virtual_cross_connect_list_params.Filter | Omit = omit,
        page: virtual_cross_connect_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectListResponse:
        """
        List all Virtual Cross Connects.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[network_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/virtual_cross_connects",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    virtual_cross_connect_list_params.VirtualCrossConnectListParams,
                ),
            ),
            cast_to=VirtualCrossConnectListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualCrossConnectDeleteResponse:
        """
        Delete a Virtual Cross Connect.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/virtual_cross_connects/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VirtualCrossConnectDeleteResponse,
        )


class VirtualCrossConnectsResourceWithRawResponse:
    def __init__(self, virtual_cross_connects: VirtualCrossConnectsResource) -> None:
        self._virtual_cross_connects = virtual_cross_connects

        self.create = to_raw_response_wrapper(
            virtual_cross_connects.create,
        )
        self.retrieve = to_raw_response_wrapper(
            virtual_cross_connects.retrieve,
        )
        self.update = to_raw_response_wrapper(
            virtual_cross_connects.update,
        )
        self.list = to_raw_response_wrapper(
            virtual_cross_connects.list,
        )
        self.delete = to_raw_response_wrapper(
            virtual_cross_connects.delete,
        )


class AsyncVirtualCrossConnectsResourceWithRawResponse:
    def __init__(self, virtual_cross_connects: AsyncVirtualCrossConnectsResource) -> None:
        self._virtual_cross_connects = virtual_cross_connects

        self.create = async_to_raw_response_wrapper(
            virtual_cross_connects.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            virtual_cross_connects.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            virtual_cross_connects.update,
        )
        self.list = async_to_raw_response_wrapper(
            virtual_cross_connects.list,
        )
        self.delete = async_to_raw_response_wrapper(
            virtual_cross_connects.delete,
        )


class VirtualCrossConnectsResourceWithStreamingResponse:
    def __init__(self, virtual_cross_connects: VirtualCrossConnectsResource) -> None:
        self._virtual_cross_connects = virtual_cross_connects

        self.create = to_streamed_response_wrapper(
            virtual_cross_connects.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            virtual_cross_connects.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            virtual_cross_connects.update,
        )
        self.list = to_streamed_response_wrapper(
            virtual_cross_connects.list,
        )
        self.delete = to_streamed_response_wrapper(
            virtual_cross_connects.delete,
        )


class AsyncVirtualCrossConnectsResourceWithStreamingResponse:
    def __init__(self, virtual_cross_connects: AsyncVirtualCrossConnectsResource) -> None:
        self._virtual_cross_connects = virtual_cross_connects

        self.create = async_to_streamed_response_wrapper(
            virtual_cross_connects.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            virtual_cross_connects.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            virtual_cross_connects.update,
        )
        self.list = async_to_streamed_response_wrapper(
            virtual_cross_connects.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            virtual_cross_connects.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FqdnListParams", "Filter", "Page"]


class FqdnListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[connection_id], filter[fqdn], filter[port],
    filter[dns_record_type]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class Filter(TypedDict, total=False):
    connection_id: str
    """ID of the FQDN connection to which the FQDN belongs."""

    dns_record_type: str
    """DNS record type used by the FQDN."""

    fqdn: str
    """FQDN represented by the resource."""

    port: int
    """Port to use when connecting to the FQDN."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import TypedDict

__all__ = ["PortoutListRejectionCodesParams", "Filter"]


class PortoutListRejectionCodesParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[code], filter[code][in]
    """


class Filter(TypedDict, total=False):
    code: Union[int, Iterable[int]]
    """Filter rejections of a specific code"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.messaging.rcs import agent_list_params, agent_update_params
from ....types.rcs_agent_response import RcsAgentResponse
from ....types.messaging.rcs.agent_list_response import AgentListResponse

__all__ = ["AgentsResource", "AsyncAgentsResource"]


class AgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AgentsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcsAgentResponse:
        """
        Retrieve an RCS agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/messaging/rcs/agents/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcsAgentResponse,
        )

    def update(
        self,
        id: str,
        *,
        profile_id: Optional[str] | Omit = omit,
        webhook_failover_url: Optional[str] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcsAgentResponse:
        """
        Modify an RCS agent

        Args:
          profile_id: Messaging profile ID associated with the RCS Agent

          webhook_failover_url: Failover URL to receive RCS events

          webhook_url: URL to receive RCS events

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/messaging/rcs/agents/{id}",
            body=maybe_transform(
                {
                    "profile_id": profile_id,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                agent_update_params.AgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcsAgentResponse,
        )

    def list(
        self,
        *,
        page: agent_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentListResponse:
        """
        List all RCS agents

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/messaging/rcs/agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"page": page}, agent_list_params.AgentListParams),
            ),
            cast_to=AgentListResponse,
        )


class AsyncAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAgentsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcsAgentResponse:
        """
        Retrieve an RCS agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/messaging/rcs/agents/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcsAgentResponse,
        )

    async def update(
        self,
        id: str,
        *,
        profile_id: Optional[str] | Omit = omit,
        webhook_failover_url: Optional[str] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcsAgentResponse:
        """
        Modify an RCS agent

        Args:
          profile_id: Messaging profile ID associated with the RCS Agent

          webhook_failover_url: Failover URL to receive RCS events

          webhook_url: URL to receive RCS events

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/messaging/rcs/agents/{id}",
            body=await async_maybe_transform(
                {
                    "profile_id": profile_id,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                agent_update_params.AgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcsAgentResponse,
        )

    async def list(
        self,
        *,
        page: agent_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentListResponse:
        """
        List all RCS agents

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/messaging/rcs/agents",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"page": page}, agent_list_params.AgentListParams),
            ),
            cast_to=AgentListResponse,
        )


class AgentsResourceWithRawResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.retrieve = to_raw_response_wrapper(
            agents.retrieve,
        )
        self.update = to_raw_response_wrapper(
            agents.update,
        )
        self.list = to_raw_response_wrapper(
            agents.list,
        )


class AsyncAgentsResourceWithRawResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.retrieve = async_to_raw_response_wrapper(
            agents.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            agents.update,
        )
        self.list = async_to_raw_response_wrapper(
            agents.list,
        )


class AgentsResourceWithStreamingResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.retrieve = to_streamed_response_wrapper(
            agents.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            agents.update,
        )
        self.list = to_streamed_response_wrapper(
            agents.list,
        )


class AsyncAgentsResourceWithStreamingResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.retrieve = async_to_streamed_response_wrapper(
            agents.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            agents.update,
        )
        self.list = async_to_streamed_response_wrapper(
            agents.list,
        )
